#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "personLib.h"
//#include "personLibWDob.h"
#include "fileUtils.h"

/*
Changelog (wrt. w03_1modular.c)

- new flow:
   1. Read new persons from the user
   2. Add new persons to persons.dat file
   3. Read all persons from persons.dat
   4. Sort, search for Gray and print all persons read in step 3

- new library fileUtils with the function file_size to determine the size of persons.dat.
- alternative commands for using personLib or personLibWDob module
*/




int main(){


    char filename[]="persons.dat";
    //char filename[]="persons2.dat";


    // read the nr of persons
    printf("\nEnter the number of new persons: ");
    size_t size;
    if(scanf("%d", &size)==1){
        if(size>0){

            //I. FILL THE LIST OF NEW PERSONS

            // declare list of persons
            struct Person nList[size];

            // populate the list
            for(size_t i = 0; i < size; i++){

                int ret = scan_person(&nList[i]);

                if(ret==0){
                    fprintf(stderr, "\nError: Wrong user input\n"); //error messages should go to the stderr output stream
                    return EXIT_FAILURE;
                }

            }


            printf("\n");

            // traverse the list
            printf("\n\n-----------------------------------\nNew persons:");
            for(size_t i = 0; i < size; i++){
                printf("\n%zu: ", i);
                print_person(nList[i]);
            }


            //II. ADD NEW PERSONS TO THE FILE

            //open the file to write
            FILE *fd = fopen(filename, "ab");
            if(fd == NULL){
                fprintf(stderr, "\nError opening the file %s\n",filename); //error messages should go to the stderr output stream
                return EXIT_FAILURE;
            }

            //write the list at once
            fwrite(nList, sizeof(nList), 1, fd);

            fclose(fd);



        }else{
            printf("\nOK. No new persons will be added\n");
        }
    }else{
        printf("\nOK. No new persons will be added\n");
    }



    //III. READ ALL PERSONS FROM THE FILE

    long sz = file_size(filename);

    size_t psize = sizeof(struct Person);

    if (sz<=0 || sz%psize!=0){
        fprintf(stderr, "\nBad file size or file not found.\n"); //error messages should go to the stderr output stream
        return EXIT_FAILURE;
    }

    size=sz/psize;

    struct Person list[size];

    FILE *fd = fopen(filename, "rb");
    if(fd == NULL){
        fprintf(stderr, "\nError opening the file %s\n",filename); //error messages should go to the stderr output stream
        return EXIT_FAILURE;
    }

    for(size_t i = 0; i < size; i++){
        fread(&list[i], psize, 1, fd);
    }

    fclose(fd);

    printf("\n-----------------------------------\nList of persons read from file:");

    for(size_t i = 0; i < size; i++){
        printf("\n%zu: ", i);
        print_person(list[i]);
    }


    //IV. SORT THE LIST

    // sort
    qsort(list, size, sizeof(struct Person), cmp_by_surname);

    // traverse the list
    printf("\n\n-----------------------------------\nPersons in the list after ordering (surname, ascending):");
    for(size_t i = 0; i < size; i++){
        printf("\n%zu: ", i);
        print_person(list[i]);
    }


    //V. BIN. SEARCH THE LIST

    // search
    struct Person key = { .surname = "Gray" };
    struct Person* result = bsearch(&key, list, size, sizeof(struct Person), cmp_by_surname);


    //print search result:

    printf("\n\n-----------------------------------\nSerching for a person with surname Gray using bsearch (search key: ");
    print_person(key);
    printf(":\n");
    if(result != NULL){
        printf("Found: ");

        print_person(*result);

    }else{
        printf("entry not found\n");
    }

    return EXIT_SUCCESS;






}
